#include <windows.h>
#include <shlobj.h>
#include "out.h"
#include "serial.h"

#define PI_VER2 "v1.0"

#ifdef __alpha
#define PI_VER PI_VER2 " (AXP)"
#else
#define PI_VER PI_VER2 " (x86)"
#endif

int getwrittentime();
void config_read();
void config_write();


char config_waveoutdir[MAX_PATH]="";
HANDLE hwavfile;
int srate, numchan, bps;
volatile int writtentime, w_offset;
Out_Module out;
static int last_pause=0;


static int CALLBACK WINAPI BrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED)
	{
	//	SetWindowText(hwnd,"Select Directory");
	//	SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_waveoutdir);

			MessageBox(hwnd,"USB Audio Plugin " PI_VER "\n"
					,"About",MB_OK);

	}
	return 0;
}


BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	if (ul_reason_for_call == DLL_PROCESS_ATTACH) DisableThreadLibraryCalls(hInst);
	return TRUE;
}



void config(HWND hwnd)
{
	char name[MAX_PATH];
	BROWSEINFO bi;
	ITEMIDLIST *idlist;
	bi.hwndOwner = hwnd;
	bi.pidlRoot = 0;
	bi.pszDisplayName = name;
	bi.lpszTitle = "Select a directory for .RAW file output:";
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;
	bi.lParam = 0;
	idlist = SHBrowseForFolder( &bi );
	if (idlist) {
		SHGetPathFromIDList( idlist, config_waveoutdir );
		config_write();
	}
}

void about(HWND hwnd)
{
	MessageBox(hwnd,"USB Audio Plugin " PI_VER "\n"
					,"About",MB_OK);
}

void init()
{
	config_read();
}

void quit()
{
}
char *scanstr_back(char *str, char *toscan, char *defval)
{
	char *s=str+strlen(str)-1;
	if (strlen(str) < 1) return defval;
	if (strlen(toscan) < 1) return defval;
	while (1)
	{
		char *t=toscan;
		while (*t)
			if (*t++ == *s) return s;
		t=CharPrev(str,s);
		if (t==s) return defval;
		s=t;
	}
}


#define ResetD	1
#define LoadSR	2
#define Escape	14

#define CPUClk	16000000

#define CMDs


void reset_dac(void)
{unsigned char temp[10], j=0;
	temp[j++]=Escape;
	temp[j++]=ResetD;
	SendString(temp,j);
}

void set_samplerate(unsigned int samplerate)
{unsigned char temp[10], j=0;
unsigned int wert;

	wert=(CPUClk/samplerate)-1;

	temp[j++]=Escape;
	temp[j++]=LoadSR;
	temp[j++]=255&(wert>>8);
	temp[j++]=wert&255;
	SendString(temp,j);
}

int open(int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms)
{
	
	Serial_Init("COM7", 19200);
	numchan = numchannels;
	srate = samplerate;

	bps = bitspersamp;

	reset_dac();
	set_samplerate(srate);	

	return 5;		//max. 5ms latency
}

void close()
{
	Uninit_Serial();
}

int write(char *buf, int len)
{
	int i,zs,j=0;
	unsigned char temp[8192];

	for (i=0; i<len/2; i++)
	{
	zs=(buf[i*2+1]<<8)|(255&buf[i*2]);
	zs&=65535;
	zs=zs>>1;

#ifdef CMDs
	if ((255&(zs>>8))==Escape)
		temp[j++]=Escape;
#endif
	temp[j++]=255&(zs>>8);

#ifdef CMDs
	if ((zs&255)==Escape)
		temp[j++]=Escape;
#endif
	temp[j++]=zs&255;

	}

	SendString(temp, j);
	return 0;
}

int canwrite()
{
	return last_pause?0:1024*1024;
}

int isplaying()
{
	return 0;
}

int pause(int pause)
{
	int t=last_pause;
	last_pause=pause;
	return t;
}

void setvolume(int volume)
{
}

void setpan(int pan)
{
}

void flush(int t)
{
  int a;
  w_offset=0;
  a = t - getwrittentime();
  w_offset=a;
}
	
int getwrittentime()
{
	int t=srate*numchan,l;
	int ms=writtentime;

	l=ms%t;
	ms /= t;
	ms *= 1000;
	ms += (l*1000)/t;

	if (bps == 16) ms/=2;

	return ms + w_offset;
}

Out_Module out = {
	OUT_VER,
	"Nullsoft RAW Writer plug-in " PI_VER,
	33,
	0, // hmainwindow
	0, // hdllinstance
	config,
	about,
	init,
	quit,
	open,
	close,
	write,
	canwrite,
	isplaying,
	pause,
	setvolume,
	setpan,
	flush,
	getwrittentime,
	getwrittentime
};

__declspec( dllexport ) Out_Module * winampGetOutModule()
{
	return &out;
}
