.include "m8def.inc"


.def voll	=r2
.def null	=r3
.def drei	=r8

.def MINL	=r4			;RAM Adresse, Start
.def MINH	=r5			;RAM Adresse, Start
.def MAXL	=r6			;RAM Adresse, Ende
.def MAXH	=r7			;RAM Adresse, Ende

.def temp	=r16

.def Nextop	=r18

.def BSizeL	=r20
.def BSizeH	=r21

;Port B
.equ RXF	=0
.equ RD		=1
.equ WS		=2
.equ MOSI	=3
.equ MISO	=4
.equ SCK	=5

;Ops
.equ LoadSRH=1
.equ LoadSRL=2

;Commands
.equ ResetD	=1
.equ LoadSR	=2
.equ Escape	=14

.equ MAX	=1120
.equ MIN	=96

.equ Samplerate=48		;48kHz

.equ CTCWert=(16000/Samplerate)-1

#define CMDs

.org 0000
rjmp reset


Reset:
ldi		r16, low(RAMEND)
out		SPL, r16			; setup stack pointer

ldi		r16, high(RAMEND)
out		SPH, r16			; setup stack pointer

ldi temp, 254				; PORTB Steuerleitungen = Ausgang
out ddrb, temp

ldi temp, 255
out PORTC, temp

out PORTD, temp

ldi temp, (1<<	SPE)|(1<<MSTR)	; Enable SPI, Master, set clock rate fck/2 
out SPCR, temp
ldi temp, (1<<SPI2X);		; 8MHz SPI Freq.
out SPSR, temp

ser temp
mov voll, temp

clr temp
mov null, temp

ldi temp, low(MAX)
mov MAXL, temp
ldi temp, high(MAX)
mov MAXH, temp

ldi temp, low(MIN)
mov MINL, temp
ldi temp, high(MIN)
mov MINH, temp

ldi temp, 3
mov drei, temp

Resetpoint:

ldi XL, low(MIN)			;Writepointer
clr XH

ldi YL, low(MIN)			;Readpointer
clr YH

clr Nextop

ldi temp, (1<<WGM12)|1;		;16 MHz Timertakt
out TCCR1B, temp

ldi temp,  high(CTCWert)
out OCR1AH, temp

ldi temp,  low(CTCWert)
out OCR1AL, temp

main:
	cp BSizeL, voll
	cpc BSizeH, drei
	brsh skipread			; Puffer voll ?
							; -> Keine Daten einlesen
	sbic PINB, RXF			; Daten verfgbar ?
  	rjmp skipread
	cbi PORTB, RD
	nop
	nop
	in temp, PIND
	sbi PORTB, RD

#ifdef CMDs
	cpi Nextop, 0			; <> 0 -> Parameter
	brne Command

	cpi temp, Escape		; Escape -> Befehl
	brne daten
	ldi Nextop, Escape
	rjmp main
#endif
daten:
	subi BSizeL, low(-1)	; 1 Byte mehr im Speicher
	sbci BSizeH, high(-1)

	st X+, temp

	cp  XL, MAXL
	cpc XH, MAXH
	brlo skipread

	ldi XL, low(MIN)
	clr XH

skipread:
	in temp, TIFR
	sbrs temp, OCF1A		; Zeit fr nchsten Samplewert ?
	rjmp main

	cpi BSizeL, 4
	cpc BSizeH, null
	brlo main				; Puffer leer ?
							; -> Keine Daten ausgeben
	out tifr, voll

	ld temp, Y+				; Wert lesen
	out SPDR, temp			; Wert ausgeben
	ld temp, Y+				; Wert lesen
Wait_Transmit1:
	sbis SPSR,SPIF
rjmp Wait_Transmit1

	out SPDR, temp			; Wert ausgeben
	ld temp, Y+				; Wert lesen
Wait_Transmit2:
	sbis SPSR,SPIF
rjmp Wait_Transmit2

cbi PORTB, WS

	out SPDR, temp			; Wert ausgeben
	ld temp, Y+				; Wert lesen
Wait_Transmit3:
	sbis SPSR,SPIF
rjmp Wait_Transmit3

	out SPDR, temp			; Wert ausgeben
	cp  YL, MAXL
	cpc YH, MAXH
	brlo skipreset
	ldi YL, low(MIN)
	clr YH
skipreset:

	subi BSizeL, low(4)		; 4 Bytes weniger im Speicher
	sbci BSizeH, high(4)

Wait_Transmit4:
	sbis SPSR,SPIF
rjmp Wait_Transmit4
sbi PORTB, WS
rjmp main



Command:
	cpi Nextop, Escape		; Escape -> Befehl holen
	breq Ops

	cpi Nextop, LoadSRH
	brne skip1
	out OCR1AH, temp
	ldi Nextop, LoadSRL
	rjmp main
skip1:
	cpi Nextop, LoadSRL
	brne skip2
	out OCR1AL, temp
	ldi Nextop, 0
	rjmp main
skip2:
	ldi Nextop, 0
rjmp main

Ops:
	ldi Nextop, 0
	cpi temp, Escape		; Doppeltes Escape -> Daten
	breq daten

	cpi temp, ResetD		; Reset DAC
	brne skipResD
	rjmp Resetpoint
skipResD:

	cpi temp, LoadSR		; Samplerate laden
	brne skipLSR
	ldi Nextop, LoadSRH
skipLSR:
rjmp main
